# -*- coding:utf-8 -*-

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.db import transaction
from django.db.models import Q
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from models import *

import datetime, time, pytz
import simplejson as json
import urlparse

from functools import wraps

import logging
logger = logging.getLogger("liver.views")

import warnings
warnings.filterwarnings(
        'error', r"DateTimeField received a naive datetime",
        RuntimeWarning, r'django\.db\.models\.fields')

################################################################################


def get_parameter(request, name):
    if request.POST:
        try:
          return request.POST[name]
        except KeyError:
          pass
    if request.GET:
        try:
          return request.GET[name]
        except KeyError as e:
          raise e

def get_token(func):
    def inner_decorator(request, *args, **kwargs):
        try:
          token = get_parameter(request, "token")
        except Exception,e:
          logger.warning("Token was not received")
          token = None
        request.token=token
        return func(request, *args, **kwargs)
    return wraps(func)(inner_decorator)


def json_response(res):
    json_responses = json.dumps(res)
    logger.debug('Query result: %s' % str(json_responses))
    return HttpResponse(json_responses,
        mimetype='application/json')


def return_error(error_code, error_message=None):
    if not error_message:
        if error_code == -500:
            error_message = "Internal Server Error"
        elif error_code == -400:
            error_message = "Bad request"
        elif error_code == -401:
            error_message = "Unauthorized"
        elif error_code == -403:
            error_message = "Forbidden"
        elif error_code == -404:
            error_message = "Not Found"
        elif error_code == -450:
            error_message = "Resource busy"
        else:
            error_message = "Unknown error"

    return (error_code, error_message)

################################################################################


@get_token
@transaction.commit_on_success
def api_external_get_worker_jobs(request):
    res = {
      "result": 0,
      "response": ""
    }

    token = request.token

    try:
      start_before = get_parameter(request, "start_before")
      start_before_date = \
          datetime.datetime.fromtimestamp(long(start_before), pytz.UTC)
    except Exception:
      start_before_date = \
          datetime.datetime.fromtimestamp(time.time()+300, pytz.UTC)

    logger.debug("start_before_date: %s" % start_before_date)

    try:
      start_after = get_parameter(request, "start_after")
      start_after_date = \
          datetime.datetime.fromtimestamp(long(start_after), pytz.UTC)
    except Exception:
      start_after_date = \
          datetime.datetime.fromtimestamp(time.time()-300, pytz.UTC)

    logger.debug("start_after_date: %s" % start_after_date)

    try:
        recorder = \
            Recorder.objects.filter(token=token)[0]
    except Exception:
        logger.error("No recorder associated to this token: %s" % token)
        result,response = return_error(-403)
        res["result"]=result
        res["response"]=response
        return json_response(res)

    try:
        job_list = RecordingJob.objects\
                .filter(enabled=True)\
                .filter(status="waiting")\
                .filter(scheduled_start_date__lte=start_before_date)\
                .filter(scheduled_start_date__gte=start_after_date)\
                .order_by("-scheduled_start_date")
        try:
            job = job_list[0]
            job.recorder = recorder
            job.status = "running"
            job.execution_date = datetime.datetime.fromtimestamp(time.time(), pytz.UTC)
            job.save()
        except Exception:
          res["result"] = -2
          res["response"] = "There is not available jobs"
          return json_response(res)

        job_dict = {}
        job_dict["id"] = job.id
        job_dict["start"] = job.scheduled_start_timestamp
        job_dict["duration"] = job.scheduled_duration
        job_dict["smil"] = job.slug() + ".smil"
        job_dict["profiles"] = []

        sg = job.sources_group
        job_dict["name"] = sg.name
        for s in sg.source_set.all():
            profile_dict = {}
            profile_dict["id"]=s.id
            profile_dict["uri"]=s.uri
            profile_dict["bitrate"]=s.bitrate
            profile_dict["name"]=\
job.slug() + "_" + str(s.id) + "_" + str(s.bitrate) + ".mp4"
            job_dict["profiles"].append(profile_dict)

        response = {}
        response[job.id] = job_dict
        res["response"] = response
        return json_response(res)

    except IOError, e:
        res["result"] = -99
        res["response"] = "Unexpected error: %s" % str(e)
        return json_response(res)

from django.core.exceptions import ObjectDoesNotExist
from django.core.exceptions import MultipleObjectsReturned

@csrf_exempt
@get_token
@transaction.commit_on_success
def api_external_notify_worker_jobs_result(request):

    submitted_jobs_result = request.body
    logger.debug("worker jobs result received: %s" % submitted_jobs_result)

    try:
      job_result_dict = json.loads(submitted_jobs_result)
    except ValueError:
        res = {}
        c,m = return_error (-400)
        res["result"] = c
        res["response"] = m
        return json_response(res)

    # >>> d['jobs']['2'].keys()
    # ['name','duration', 'start', 'id', 'profiles', 'result']
    # >>> d['jobs']['2']['profiles'][0].keys()
    # ['job_start', 'job_duration', 'name', 'duration', 'id', 'destination', 'uri', 'job_id', 'result']

    for k,job_dict in job_result_dict["jobs"].iteritems():
        try:
            job = RecordingJob.objects.get(id=job_dict["id"],
                recorder__token=request.token)
            job.completion_date = datetime.datetime.fromtimestamp(time.time(), pytz.UTC)
            job.result=job_dict["result"]
            if job_dict['result'] != 0:
                job.status="failed"
                logger.error("Recording job %s failed" % (job))
                job.save()
            else:
                job.status="successful"
                job.save()

                r = Recording()
                r.recording_job=job
                r.recorder=job.recorder
                r.name="%(name)s-%(start)s-%(duration)s" % (job_dict)
                r.profiles_json=json.dumps(job_dict['profiles'])
                metadata_list = []
                for m in job.recordingjobmetadata_set.all():
                    metadata_list.append({m.key:m.value})
                metadata_list.append({"start_date":str(job.scheduled_start_date)})
                metadata_list.append({"start_timestamp":job.scheduled_start_timestamp})
                metadata_list.append({"duration":job.scheduled_duration})
                metadata_list.append({"smil":job_dict["smil"]})

                r.metadata_json=json.dumps(metadata_list)
                r.save()

        except ObjectDoesNotExist:
            logger.error("Job id %s doesn't exist" % (job["id"]))
        except MultipleObjectsReturned:
            logger.error("Multiple selectables objects with id %s" % (job["id"]))
        except Exception  as e:
            logger.error("Unexpected exception saving a job result: %s" % (e))


    res = {
            "result": 0,
            "response": "Job status updated"
    }
    return json_response(res)


@get_token
@transaction.commit_on_success
def api_external_get_mo_to_delete(request):
    try:
        res = {
          "result": 0,
          "response": ""
        }

        token = request.token

        try:
            recorder = \
                Recorder.objects.filter(token=token)[0]
        except Exception:
            logger.error("No recorder associated to this token: %s" % token)
            result,response = return_error(-403)
            res["result"]=result
            res["response"]=response
            return json_response(res)

        recordings_to_delete_list = Recording.objects\
            .filter(to_delete=True).iterator()

        response = []
        for r in recordings_to_delete_list:
            try:
                profiles = json.loads(r.profiles_json)
                logger.info("Deleting recording %s" % (r))
                logger.debug("Deleting profiles for %s : %s" % (r,r.profiles_json))
                for p in profiles:
                    response.append(p["destination"])
                r.delete()
            except Exception as e:
                logger.error( "Exception occurs getting profiles for %s: %s"% (r,e))
        res["response"]=response
        return json_response(res)

    except Exception as e:
      res["result"] = -9
      res["response"] = "Unexpected error: %s" % str(e)
      return json_response(res)


@get_token
@transaction.commit_on_success
def api_external_get_mo(request):
    try:
        res = {
          "result": 0,
          "response": ""
        }

        token = request.token

        all_mo = 0
        try:
            all_mo = get_parameter(request, "all")
            all_mo = int(all_mo)
        except Exception:
            pass

        try:
            recorder = \
                Recorder.objects.filter(token=token)[0]
        except Exception:
            logger.error("No recorder associated to this token: %s" % token)
            result,response = return_error(-403)
            res["result"]=result
            res["response"]=response
            return json_response(res)



        if all_mo == 1:
            recordings_list = Recording.objects.iterator()
        else:
            recordings_list = Recording.objects\
                .filter(to_delete=False).iterator()

        response = []
        for r in recordings_list:
            try:
                profiles = json.loads(r.profiles_json)
                logger.debug("Getting profiles for %s : %s" % (r,r.profiles_json))
                for p in profiles:
                    response.append(p["destination"])
                metadatas = json.loads(r.metadata_json)
                if all_mo == 1:
                    logger.debug("Getting smil for %s : %s" % (r,r.metadata_json))
                    for m in metadatas:
                        try:
                            k = m.keys()[0]
                            v = m.values()[0]
                            if k == "smil":
                                response.append(v)
                        except Exception,e:
                            logger.critical(e)
                            pass

            except Exception as e:
                logger.error( "Exception occurs getting profiles for %s: %s"% (r,e))
        res["response"]=response
        return json_response(res)

    except Exception as e:
      res["result"] = -9
      res["response"] = "Unexpected error: %s" % str(e)
      return json_response(res)

@csrf_exempt
@get_token
@transaction.commit_on_success
def api_external_update_recordings(request):
    try:
        res = {
          "result": 0,
          "response": ""
        }

        token = request.token

        now = int(time.time())
        now_date = datetime.datetime.fromtimestamp(now,pytz.UTC)

        try:
            application = Application.objects.filter(token=token)\
                 .filter(valid=True)\
                 .filter( Q(valid_since__lt=now_date)\
                                        | Q(valid_since__isnull=True) ) \
                 .filter( Q(valid_until__gt=now_date)\
                                        | Q(valid_until__isnull=True) )[0]

        except Exception, e:
            logger.debug("Exception getting application: %s", e)
            logger.error("No application associated to this token: %s" % token)
            result,response = return_error(-403)
            res["result"]=result
            res["response"]=response
            return json_response(res)

        submitted_updatings = request.body
        logger.debug("update_recordings received: %s" % submitted_updatings)

        try:
            updatings = json.loads(submitted_updatings)
        except ValueError:
            res = {}
            c,m = return_error (-400)
            res["result"] = c
            res["response"] = m
            return json_response(res)

        for u in updatings:
            updating_problem=True
            logger.info("Request for updating recordings: %s" % u)
            recordings = None
            if u.has_key("database_key"):
                recordings = Recording.objects.filter(\
metadata_json__contains='{"database_key": "%s"}'%u["database_key"].strip())
            else:
                if u.has_key("event_id"):
                    recordings = Recording.objects.filter(\
metadata_json__contains='{"event_id": "%s"}'%u["event_id"].strip())

            recording_jobs = []
            if u.has_key("database_key"):
                r_j_m_list = RecordingJobMetadata.objects.filter(\
key="database_key",value=u["database_key"].strip()).iterator()
                for r_j_m in r_j_m_list:
                    recording_jobs.append(r_j_m.recording_job)
            else:
                if u.has_key("event_id"):
                    r_j_m_list = RecordingJobMetadata.objects.filter(\
key="event_id",value=u["event_id"].strip()).iterator()
                    for r_j_m in r_j_m_list:
                        recording_jobs.append(r_j_m.recording_job)

            for k,v in u.iteritems():
                # Updating Recording
                try:
                    if recordings:
                        logger.info("Updating recordings")
                        for recording in recordings:
                            metadatas = json.loads(recording.metadata_json)
                            for m in metadatas:
                                for k1,v1 in m.iteritems():
                                    if k1 == k:
                                        logger.info(\
"Recording %s updated. %s = %s" % (recording,k1,v))
                                        m[k1] = v
                            recording.metadata_json = json.dumps(metadatas)
                            recording.save()
                            updating_problem = False
                except Exception, e:
                    logger.debug(\
"Unexpected updating Recording values: %s" % e)

                # Updating RecordingJobMetadata
                try:
                    if len(recording_jobs)>0:
                        logger.info("Updating recording_jobs")
                        for recording_job in recording_jobs:
                            r_j_m = RecordingJobMetadata.objects.filter(\
recording_job=recording_job,key=k).update(value=v.strip())
                            logger.info(\
"RecorderJob %s updated. %s = %s" % (recording_job,k,v))
                            updating_problem = False
                except Exception, e:
                    logger.debug(\
"Unexpected updating RecordingJobMetadata values: %s" % e)

            if updating_problem:
                res = {
                    "result": -1,
                    "response": "Parcial errors"
                }

    except Exception  as e:
        logger.error("Unexpected exception updating recordings info: %s" % (e))
        res = {
            "result": -1,
            "response": "Unexpected exception updating recordings info: %s" % (e)
        }

    return json_response(res)








from django.db import models
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy, ugettext as _

import datetime, time, pytz, calendar
import simplejson as json
import uuid

class SourcesGroup(models.Model):
    name = models.CharField(max_length=5000,verbose_name="Common Name")
    external_id = models.CharField(max_length=5000,verbose_name="External Id.")

    insertion_date = models.DateTimeField(editable=False)
    modification_date = models.DateTimeField(auto_now=True,
            auto_now_add=True, blank=True, null=True)
    default_offset_start = models.IntegerField(null=True, blank=True,
            verbose_name="Default offset start (in seconds)")
    default_offset_end = models.IntegerField(null=True, blank=True,
            verbose_name="Default offset end (in seconds)")
    default_availability_window = models.IntegerField(null=True, blank=True,
            verbose_name="Default licensing window (in hours)")

    def clone(self):
        sg = SourcesGroup()
        if self.name and self.name.find(" (Clone") >= 0:
            sg.name = self.name[:self.name.find(" (Clone")] \
                    + " (Clone %s)" % int(time.time())
        else:
            sg.name = self.name + " (Clone %s)" % int(time.time())
        sg.external_id = self.external_id
        sg.default_offset_start = self.default_offset_start
        sg.default_offset_end = self.default_offset_end
        sg.default_availability_window = self.default_availability_window

        sg.save()

        for s in self.source_set.all():
            s_clone = s.clone()
            s_clone.sources_group = sg
            s_clone.save()

        return sg

    def save(self, *args, **kwargs):
        if not self.insertion_date:
            d = datetime.datetime.fromtimestamp(time.time(), pytz.UTC)
            self.insertion_date = d
        super(SourcesGroup, self).save(*args, **kwargs)

    def __repr__(self):
        return self.__unicode__()

    def __unicode__(self):
        return u"%s [%s]" % (slugify(self.name),
                self.external_id)



class Source(models.Model):
    name = models.CharField(max_length=5000,verbose_name="Common Name")
    uri = models.CharField(max_length=5000,verbose_name="URI")
    bitrate = models.IntegerField(verbose_name="Bitrate (bps)",
            default=1000000)

    insertion_date = models.DateTimeField(editable=False)
    modification_date = models.DateTimeField(auto_now=True,
            auto_now_add=True, blank=True, null=True)

    sources_group = models.ForeignKey(SourcesGroup, null=True, blank=True)

    def clone(self):
        s = Source()
        if self.name and self.name.find(" (Clone") >= 0:
            s.name = self.name[:self.name.find(" (Clone")] \
                    + " (Clone %s)" % int(time.time())
        else:
            s.name = self.name + " (Clone %s)" % int(time.time())
        s.bitrate = self.bitrate
        s.uri = self.uri
        s.save()
        return s

    def save(self, *args, **kwargs):
        if not self.insertion_date:
            d = datetime.datetime.fromtimestamp(time.time(), pytz.UTC)
            self.insertion_date = d
        super(Source, self).save(*args, **kwargs)

    def __repr__(self):
        return self.__unicode__()

    def __unicode__(self):
        return u"%s [%s] [%s]" % (slugify(self.name), self.uri,self.bitrate)


class Recorder(models.Model):
    name = models.CharField(max_length=5000)
    token = models.CharField(max_length=5000,default=str(uuid.uuid1()))

    def clone(self):
        r = Recorder()
        if self.name and self.name.find(" (Clone") >= 0:
            r.name = self.name[:self.name.find(" (Clone")] \
                    + " (Clone %s)" % int(time.time())
        else:
            r.name = self.name + " (Clone %s)" % int(time.time())
        r.token = self.token
        r.save()
        return r

    def __repr__(self):
        return self.__unicode__()

    def __unicode__(self):
        return "%s [%s]" % (self.name, self.token)

class RecordingSource(models.Model):
    external_id = models.CharField(max_length=5000,verbose_name="External Id.",
            default = "")
    sources_group = models.ForeignKey(SourcesGroup, null=True, blank=True)

    insertion_date = models.DateTimeField(editable=False)
    modification_date = models.DateTimeField(auto_now=True,
            auto_now_add=True, blank=True, null=True)
    enabled = models.BooleanField(default=True)
    enabled_since = models.DateTimeField(null=True, blank=True)
    enabled_until = models.DateTimeField(null=True, blank=True,
            default = None, verbose_name="Enabled to")

    def clone(self):
        rs = RecordingSource()
        rs.external_id = self.external_id
        rs.sources_group = self.sources_group
        rs.enabled = False
        rs.enabled_since = self.enabled_since
        rs.enabled_until = self.enabled_until
        rs.save()
        rm_list = RecordingMetadata.objects.filter(recording_source=self)
        for rm in rm_list:
            rm_clone = rm.clone()
            rm_clone.recording_source=rs
            rm_clone.save()
        rr_list = RecordingRule.objects.filter(recording_source=self)
        for rr in rr_list:
            rr_clone = rr.clone()
            rr_clone.recording_source=rs
            rr_clone.save()
        return rs

    def save(self, *args, **kwargs):
        if not self.insertion_date:
            d = datetime.datetime.fromtimestamp(time.time(), pytz.UTC)
            self.insertion_date = d
        super(RecordingSource, self).save(*args, **kwargs)

    def __repr__(self):
        return self.__unicode__()

    def __unicode__(self):
        return "Recording %s" % (unicode(self.sources_group))


class RecordingMetadata(models.Model):
    recording_source = models.ForeignKey(RecordingSource)

    key = models.CharField(max_length=5000,blank=True)
    value = models.CharField(max_length=5000,blank=True)

    def clone(self):
        rm = RecordingMetadata()
        rm.recording_source = self.recording_source
        rm.key = self.key
        rm.value = self.value
        rm.save()
        return rm

    def __repr__(self):
        return self.__unicode__()

    def __unicode__(self):
        return "%s - %s" % (self.key, self.value)


class RecordingRule(models.Model):
    recording_source = models.ForeignKey(RecordingSource)

    metadata_key_filter = models.CharField(max_length=5000,blank=True)
    metadata_value_filter = models.CharField(max_length=5000,blank=True)

    offset_start = models.IntegerField(null=True, blank=True,
            verbose_name="Offset start (in seconds)")
    offset_end = models.IntegerField(null=True, blank=True,
            verbose_name="Offset end(in seconds)")
    availability_window = models.IntegerField(null=True, blank=True,
            verbose_name="Licensing window (in hours)")

    def clone(self):
        rr = RecordingRule()
        rr.recording_source = self.recording_source
        rr.metadata_key_filter = self.metadata_key_filter
        rr.metadata_value_filter = self.metadata_value_filter
        rr.offset_start = self.offset_start
        rr.offset_end = self.offset_end
        rr.availability_window = self.availability_window
        rr.save()
        return rr

    def __repr__(self):
        return self.__unicode__()

    def __unicode__(self):
        return "[%s:%s] [start:%s,end:%s] (%s)" \
    % (self.metadata_key_filter,
       self.metadata_value_filter,
       self.offset_start,
       self.offset_end,
       self.availability_window)


class RecordingJob(models.Model):
    class Meta:
        verbose_name = _('Recording job')

    status_choices = [
        ('waiting', _('Waiting')),
        ('running', _('Running')),
        ('failed', _('Failed')),
        ('successful', _('Successful')),
        ('cancelled', _('Cancelled')),
    ]

    recording_source = models.ForeignKey(RecordingSource, null=True, blank=True,
                        on_delete=models.SET_NULL)

    sources_group = models.ForeignKey(SourcesGroup)

    insertion_date = models.DateTimeField(editable=False)
    modification_date = models.DateTimeField(auto_now=True,
            auto_now_add=True, blank=True, null=True)
    execution_date = models.DateTimeField(null=True, blank=True)
    completion_date = models.DateTimeField(null=True, blank=True)

    scheduled_start_date = models.DateTimeField(null=True, blank=True)
    scheduled_end_date = models.DateTimeField(null=True, blank=True,
            editable=False)
    scheduled_duration = models.IntegerField(
            verbose_name="Duration (sec.)")

    enabled = models.BooleanField(default=True)

    recorder =  models.ForeignKey(Recorder, null=True, blank=True,
            on_delete=models.SET_NULL)

    result = models.CharField(max_length=5000, blank=True, null=True,
            verbose_name=_('Result'),default="None")
    status = models.CharField(max_length=5000,
            verbose_name=_('Status'),
            choices=status_choices)

    def get_scheduled_start_timestamp(self):
        return calendar.timegm(
            self.scheduled_start_date.astimezone(
                pytz.utc).utctimetuple())
    scheduled_start_timestamp = property(get_scheduled_start_timestamp)

    def clone(self):
        rj = RecordingJob()
        rj.recording_source = self.recording_source
        rj.sources_group = self.sources_group
        rj.scheduled_start_date = self.scheduled_start_date
        rj.scheduled_duration = self.scheduled_duration
        rj.enabled = self.enabled
        rj.save()
        rjm_list = RecordingJobMetadata.objects.filter(recording_job=self)
        for rjm in rjm_list:
            rjm_clone = rjm.clone()
            rjm_clone.recording_job=rj
            rjm_clone.save()
        return rj

    def save(self, *args, **kwargs):
        if not self.insertion_date:
            d = datetime.datetime.fromtimestamp(time.time(), pytz.UTC)
            self.insertion_date = d

        try:
            _timestamp = \
calendar.timegm(self.scheduled_start_date.astimezone(pytz.utc).utctimetuple())\
+self.scheduled_duration
            self.scheduled_end_date = datetime.datetime.fromtimestamp(_timestamp,pytz.UTC)
        except Exception, e:
            pass

        super(RecordingJob, self).save(*args, **kwargs)

    def __repr__(self):
        return self.__unicode__()

    def __unicode__(self):
        return "[%s] %s [start:%s, duration:%s]" \
    % (self.id,
       self.sources_group,
       self.scheduled_start_date,
       self.scheduled_duration)

    def slug(self):
        return slugify("%s-%s-%s-%s" \
    % (self.sources_group,
       self.scheduled_start_date,
       self.scheduled_duration,
       self.id))

    def pretty_name(self):
        try:
            title = self.recordingjobmetadata_set.filter(key="title")[0]
        except Exception:
            title = "-"
        return "[%s] %s [%s]" \
    % (self.id,
       self.sources_group,
       title)



class RecordingJobMetadata(models.Model):
    recording_job = models.ForeignKey(RecordingJob)

    key = models.CharField(max_length=5000,blank=True)
    value = models.CharField(max_length=5000,blank=True)

    def clone(self):
        rjm = RecordingJobMetadata()
        rjm.recording_job = self.recording_job
        rjm.key = self.key
        rjm.value = self.value
        rjm.save()
        return rjm

    def __repr__(self):
        return self.__unicode__()

    def __unicode__(self):
        return "%s - %s" % (self.key, self.value)


class Recording(models.Model):
    recording_job = models.ForeignKey(RecordingJob, on_delete=models.SET_NULL,
            null=True, blank=True,
            verbose_name="Associated recording job")
    recorder = models.ForeignKey(Recorder, on_delete=models.SET_NULL,
            null=True, blank=True,
            verbose_name="Associated recorder")

    name = models.CharField(max_length=5000,blank=True,editable=False)

    insertion_date = models.DateTimeField(editable=False)
    modification_date = models.DateTimeField(auto_now=True,
            auto_now_add=True, blank=True, null=True)

    metadata_json = models.TextField(max_length=5000,blank=True,editable=False) # json
    def get_metadata(self):
        try:
          res = ""
          m_list = json.loads(self.metadata_json)
          for m in m_list:
              res +=\
"%s:%s\n" % (m.keys()[0], m.values()[0])
        except Exception:
          res = self.metadata_json
        return res
    metadata = property(get_metadata)

    profiles_json = models.TextField(max_length=5000,blank=True,editable=False) # json
    def get_profiles(self):
        try:
          res = ""
          p_list = json.loads(self.profiles_json)
          for p in p_list:
              res +=\
"uri:%(uri)s file:%(destination)s\n"  % p

        except Exception:
          res = self.profiles_json
        return res
    profiles = property(get_profiles)

    to_delete = models.BooleanField(default=False)

    def save(self, *args, **kwargs):
        if not self.insertion_date:
            d = datetime.datetime.fromtimestamp(time.time(), pytz.UTC)
            self.insertion_date = d
        # TODO: Implementar la logica que decide si el flag to_delete se
        # activa o no
        super(Recording, self).save(*args, **kwargs)

    def delete(self, using=None):
        if self.to_delete:
            return super(Recording, self).delete()
        else:
            self.to_delete = True
            self.save()

    def clone(self):
        r = Recording()
        if self.name and self.name.find(" (Clone") >= 0:
            r.name = self.name[:self.name.find(" (Clone")] \
                    + " (Clone %s)" % int(time.time())
        else:
            r.name = self.name + " (Clone %s)" % int(time.time())

        r.metadata_json = self.metadata_json
        r.profiles_json = self.profiles_json
        r.recording_job = self.recording_job
        r.save()
        return r

    def __repr__(self):
        return self.__unicode__()

    def __unicode__(self):
        return "%s" % (self.name)


class Application(models.Model):

    class Meta:
        verbose_name = 'External application'

    name = models.CharField(max_length=250, blank=True)
    description = models.TextField(max_length=2500, blank=True)

    token = models.CharField(max_length=500, blank=True)

    insertion_time = models.DateTimeField(auto_now_add=True, null=True, blank=True, db_index=True)
    modification_time = models.DateTimeField(auto_now=True,
            auto_now_add=True, blank=True, db_index=True, null=True)
    valid = models.BooleanField(default=False)
    valid_since = models.DateTimeField(null=True, blank=True, db_index=True)
    valid_until = models.DateTimeField(null=True, blank=True, db_index=True,
            default = None)
    sync_time = models.DateTimeField(blank=True, db_index=True, null=True )


    def __unicode__(self):
        return "%s - %s" % (self.name,self.token)
    def save(self, *args, **kwargs):
        d = datetime.datetime.fromtimestamp(time.time(), pytz.UTC)
        self.modification_time = d
        if not self.insertion_time:
            self.insertion_time = d
        if not self.token or len(self.token) == 0:
            self.token = str(uuid.uuid1())
        self.token = self.token.strip()

        super(Application, self).save(*args, **kwargs)






from models import *

import logging
logger = logging.getLogger("liver.admin")

from django import forms
from django.contrib import admin
from django.db.models import Q

from django.utils.translation import ugettext_lazy, ugettext as _

import datetime, time, pytz, calendar

def clone(modeladmin, request, queryset):
    for o in queryset:
        o.clone()
clone.short_description = _("Clone")

class RecordingRuleInLine(admin.TabularInline):
        model = RecordingRule
        extra = 1

class RecordingMetadataInLine(admin.TabularInline):
        model = RecordingMetadata
        extra = 1

class RecordingJobMetadataInLine(admin.TabularInline):
        model = RecordingJobMetadata
        extra = 1

class RecordingSourceAdmin(admin.ModelAdmin):
    actions = [
            clone,
    ]

    inlines = [
                RecordingRuleInLine,
                RecordingMetadataInLine,
    ]

    list_display = [
            'sources_group',
            # 'insertion_date',
            'modification_date',
            'enabled',
            'enabled_since',
            'enabled_until',
            'edit_html',
    ]

    list_editable = [
            'sources_group',
            'enabled',
            'enabled_since',
            'enabled_until',

    ]

    list_display_links = ['edit_html']

    list_per_page = 200

    def edit_html(self, queryset):
        return '''<a href="%s/">Edit</a>''' % queryset.id
    edit_html.short_description = ''
    edit_html.allow_tags = True



class RecordingJobAdmin(admin.ModelAdmin):
    ordering = ['-scheduled_start_date']

    actions = [
            "wait",
            "cancel",
            "success",
            "launch_now",
            clone,
    ]

    def launch_now(self, request, queryset):
        d = datetime.datetime.fromtimestamp(time.time(), pytz.UTC)
        for q in queryset:
            q.status="waiting"
            q.scheduled_start_date=d
            q.save()
    launch_now.short_description=_("Launch now")

    def wait(self, request, queryset):
        queryset.update(status="waiting")
    wait.short_description=_("Set to waiting")

    def cancel(self, request, queryset):
        queryset.update(status="cancelled")
    cancel.short_description=_("Set to cancelled")

    def success(self, request, queryset):
        queryset.update(status="successful")
    success.short_description=_("Set to successful")

    date_hierarchy = 'scheduled_start_date'

    list_per_page = 200

    search_fields = ['status','recordingjobmetadata__value' ]

    exclude = ["recording_source"]
    readonly_fields = [
            'insertion_date',
            'modification_date',
            'execution_date',
            'completion_date',
            'scheduled_end_date',
    ]

    fieldsets = (
        (None, {
            'fields': (
                (
            'sources_group',
            'scheduled_start_date',
            'scheduled_end_date',
            'scheduled_duration',
            'enabled',
                ),

            )
        }),
        (_("Process"), {
            'fields': (
                (
            'status',
            'recorder',
            'result',
                ),
                (
            'insertion_date',
            'modification_date',
            'execution_date',
            'completion_date',
                ),

            )
        }),

    )

    list_display = [
      'pretty_name',
      'scheduled_start_date',
      'scheduled_end_date',
      'scheduled_duration',
      'insertion_date',
      'modification_date',
      'execution_date',
      'status',
    ]

    list_filter = [
            "enabled",
            "status",
            "recording_source",
            "recorder",
    ]

    inlines = [
        RecordingJobMetadataInLine,
    ]

class RecordingAdmin(admin.ModelAdmin):
    search_fields = ['name', 'metadata_json', 'profiles_json']

    ordering = ['-insertion_date']

    actions = [
            # clone,
    ]

    readonly_fields = [
            'name',
            'insertion_date',
            'modification_date',
            'recording_job',
            'profiles',
            'metadata',
    ]

    fieldsets = (
        (None, {
            'fields': (
                (
            'name',
            'insertion_date',
            'modification_date',
                ),

            )
        }),
        (_("Information"), {
            'fields': (
                (
            'profiles',
            'metadata',
                ),
            )
        }),

    )


    list_display = [
            'name',
            'insertion_date',
            'modification_date',
            'profiles',
            'metadata',
    ]

    list_per_page = 200

    def has_add_permission(self, request):
        return False

    def queryset(self, request):
        qs = super(RecordingAdmin, self).queryset(request)
        return qs.filter(Q(to_delete=False))


class SourceInLine(admin.TabularInline):
    model = Source
    fk_name = 'sources_group'
    extra = 0

class SourcesGroupAdmin(admin.ModelAdmin):
    actions = [
            clone,
    ]
    ordering = ['name']

    readonly_fields = [
            'insertion_date',
            'modification_date',
    ]

    fieldsets = (
        (None, {
            'fields': (
                (
            'name',
            'external_id',
                ),

            )
        }),
        (None, {
            'fields': (
                (
            'insertion_date',
            'modification_date',
                ),

            )
        }),
        (None, {
            'fields': (
                (
            'default_offset_start',
            'default_offset_end',
            'default_availability_window',
                ),

            )
        }),
    )

    inlines = [
      SourceInLine,
    ]


class RecorderAdmin(admin.ModelAdmin):
    ordering = ['name','token']


class ApplicationAdmin(admin.ModelAdmin):

    readonly_fields = [
            'id',
            'sync_time',
    ]

    list_editable = ['name','token', 'valid', 'valid_since','valid_until']

    list_display_links = ['edit_html']
    list_display = ('name', 'token', 'description',
            'valid', 'valid_since', 'valid_until',
            'modification_time', "edit_html")
    # list_filter = ['valid', 'source_host','enabled',
    #         ]
    search_fields = ['name', 'description','token',
            ]
    date_hierarchy = 'insertion_time'
    list_per_page = 200

    fieldsets = [
            (None,{
              'fields': [
                ('name','token'),
                ('valid','valid_since', 'valid_until','sync_time'),
                (),
            ]}),
            ('Other info', {
              'classes': ('collapse',),
              'fields': (
                ("description"),
              )
            }),
    ]

    def edit_html(self, queryset):
        return '''<a href="%s/">Edit</a>''' % queryset.id
    edit_html.short_description = ''
    edit_html.allow_tags = True



# admin.site.register(Source)
admin.site.register(SourcesGroup,SourcesGroupAdmin)
admin.site.register(Recorder,RecorderAdmin)
admin.site.register(RecordingSource,RecordingSourceAdmin)
admin.site.register(RecordingJob,RecordingJobAdmin)
admin.site.register(Recording,RecordingAdmin)
admin.site.register(Application,ApplicationAdmin)




from setuptools import setup, find_packages

version = "0.0.35"

long_description=""
try:
    long_description=file('README.md').read()
except Exception:
    pass

license=""
try:
    license=file('MIT_License.txt').read()
except Exception:
    pass

setup(
    name = 'liver',
    version = version,
    description = 'Video recorder',
    author = 'Pablo Saavedra',
    author_email = 'pablo.saavedra@interoud.com',
    url = 'http://github.com/psaavedra/liver',
    download_url= 'https://github.com/psaavedra/liver/zipball/master',
    packages = find_packages(),
    package_data={
    },
    scripts=[
        "tools/",
        "tools/liver-delete-jobs",
        "tools/liver-delete-recordings",
        "tools/liver-do-jobs",
        "tools/liver-get-mo-to-delete",
        "tools/liver-mo-status"
    ],
    zip_safe=False,
    install_requires=[
        "django >=1.6, <1.7",
        "python-dateutil",
        "httplib2",
        "simplejson",
        "slugify",
    ],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Framework :: Django",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Multimedia :: Video",
    ],
    long_description=long_description,
    license=license,
    keywords = "rule manager django liver recorder",
)
